// SPDX-License-Identifier: MIT
// Copyright contributors to the kepler.gl project

import esbuild from "esbuild";

import process from "node:process";
import fs from "node:fs";
import { spawn } from "node:child_process";
import { join } from "node:path";

const args = process.argv;

const port = 8081;

const config = {
  platform: "browser",
  format: "iife",
  logLevel: "info",
  loader: { ".js": "jsx", ".css": "css" },
  entryPoints: ["src/app.tsx"],
  outfile: "dist/bundle.js",
  bundle: true,
  define: {
    NODE_ENV: JSON.stringify(process.env.NODE_ENV || "production"),
    "process.env.MapboxAccessToken": JSON.stringify(
      process.env.MapboxAccessToken || ""
    ),
    "process.env.DropboxClientId": JSON.stringify(
      process.env.DropboxClientId || ""
    ),
    "process.env.MapboxExportToken": JSON.stringify(
      process.env.MapboxExportToken || ""
    ),
    "process.env.CartoClientId": JSON.stringify(
      process.env.CartoClientId || ""
    ),
    "process.env.FoursquareClientId": JSON.stringify(
      process.env.FoursquareClientId || ""
    ),
    "process.env.FoursquareDomain": JSON.stringify(
      process.env.FoursquareDomain || ""
    ),
    "process.env.FoursquareAPIURL": JSON.stringify(
      process.env.FoursquareAPIURL || ""
    ),
    "process.env.FoursquareUserMapsURL": JSON.stringify(
      process.env.FoursquareUserMapsURL || ""
    ),
    "process.env.OpenAIToken": JSON.stringify(process.env.OpenAIToken || ""),
    "process.env.NODE_DEBUG": JSON.stringify(false),
  },
  plugins: [],
};

function openURL(url) {
  const cmd = {
    darwin: ["open"],
    linux: ["xdg-open"],
    win32: ["cmd", "/c", "start"],
  };
  const command = cmd[process.platform];
  if (command) {
    spawn(command[0], [...command.slice(1), url]);
  }
}

(async () => {
  if (args.includes("--build")) {
    await esbuild
      .build({
        ...config,

        minify: true,
        sourcemap: false,
      })
      .catch((e) => {
        console.error(e);
        process.exit(1);
      });
  }

  if (args.includes("--start")) {
    await esbuild
      .context({
        ...config,
        minify: false,
        sourcemap: true,
        // add alias to resolve libraries so there is only one copy of them
        ...(process.env.NODE_ENV === "local" ? { alias: localAliases } : {}),
        banner: {
          js: `new EventSource('/esbuild').addEventListener('change', () => location.reload());`,
        },
      })
      .then(async (ctx) => {
        await ctx.watch();
        await ctx.serve({
          servedir: "dist",
          port,
          fallback: "dist/index.html",
          onRequest: ({ remoteAddress, method, path, status, timeInMS }) => {
            console.info(
              remoteAddress,
              status,
              `"${method} ${path}" [${timeInMS}ms]`
            );
          },
        });
        console.info(
          `kepler.gl demo app running at ${`http://localhost:${port}`}, press Ctrl+C to stop`
        );
        openURL(`http://localhost:${port}`);
      })
      .catch((e) => {
        console.error(e);
        process.exit(1);
      });
  }
})();
